@@ -34,6 +34,9 @@ class TMWSHARED_EXPORT TmwPlugin : public QObject,
 {
     Q_OBJECT
     Q_INTERFACES(Tiled::MapWriterInterface)
+#if QT_VERSION >= 0x050000
+    Q_PLUGIN_METADATA(IID "org.mapeditor.MapWriterInterface" FILE "plugin.json")
+#endif
 
 public:
     TmwPlugin();
