@@ -23,6 +23,7 @@
 #include "commandlineparser.h"
 #include "mainwindow.h"
 #include "languagemanager.h"
+#include "pluginmanager.h"
 #include "preferences.h"
 #include "tiledapplication.h"
 
@@ -85,8 +86,7 @@ CommandLineHandler::CommandLineHandler()
     option<&CommandLineHandler::justQuit>(
                 QChar(),
                 QLatin1String("--quit"),
-                QLatin1String("Only check validity of arguments, "
-                              "don't actually load any files"));
+                QLatin1String("Only check validity of arguments"));
 
     option<&CommandLineHandler::setDisableOpenGL>(
                 QChar(),
@@ -98,7 +98,7 @@ void CommandLineHandler::showVersion()
 {
     if (!showedVersion) {
         showedVersion = true;
-        qWarning() << "Tiled (Qt) Map Editor"
+        qWarning() << qPrintable(QApplication::applicationName())
                    << qPrintable(QApplication::applicationVersion());
         quit = true;
     }
@@ -133,7 +133,7 @@ int main(int argc, char *argv[])
 #ifdef BUILD_INFO_VERSION
     a.setApplicationVersion(QLatin1String(AS_STRING(BUILD_INFO_VERSION)));
 #else
-    a.setApplicationVersion(QLatin1String("0.8.1"));
+    a.setApplicationVersion(QLatin1String("0.9.0"));
 #endif
 
 #ifdef Q_WS_MAC
@@ -152,6 +152,8 @@ int main(int argc, char *argv[])
     if (commandLine.disableOpenGL)
         Preferences::instance()->setUseOpenGL(false);
 
+    PluginManager::instance()->loadPlugins();
+
     MainWindow w;
     w.show();
 
