@@ -24,6 +24,7 @@
 #ifndef MAPSCENE_H
 #define MAPSCENE_H
 
+#include <QColor>
 #include <QGraphicsScene>
 #include <QMap>
 #include <QSet>
@@ -108,17 +109,6 @@ public:
 signals:
     void selectedObjectItemsChanged();
 
-public slots:
-    /**
-     * Sets whether the tile grid is visible.
-     */
-    void setGridVisible(bool visible);
-
-    /**
-     * Sets whether the current layer should be highlighted.
-     */
-    void setHighlightCurrentLayer(bool highlightCurrentLayer);
-
 protected:
     /**
      * QGraphicsScene::drawForeground override that draws the tile grid.
@@ -137,6 +127,18 @@ protected:
     void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
 
 private slots:
+    /**
+     * Sets whether the tile grid is visible.
+     */
+    void setGridVisible(bool visible);
+
+    void setShowTileObjectOutlines(bool enabled);
+
+    /**
+     * Sets whether the current layer should be highlighted.
+     */
+    void setHighlightCurrentLayer(bool highlightCurrentLayer);
+
     /**
      * Refreshes the map scene.
      */
@@ -174,12 +176,14 @@ private:
     AbstractTool *mSelectedTool;
     AbstractTool *mActiveTool;
     bool mGridVisible;
+    bool mShowTileObjectOutlines;
     bool mHighlightCurrentLayer;
     bool mUnderMouse;
     Qt::KeyboardModifiers mCurrentModifiers;
     QPointF mLastMousePos;
     QVector<QGraphicsItem*> mLayerItems;
     QGraphicsRectItem *mDarkRectangle;
+    QColor mDefaultBackgroundColor;
 
     typedef QMap<MapObject*, MapObjectItem*> ObjectItems;
     ObjectItems mObjectItems;
