@@ -26,6 +26,9 @@
 
 #include <QApplication>
 #include <QCursor>
+#include <QGesture>
+#include <QGestureEvent>
+#include <QPinchGesture>
 #include <QWheelEvent>
 #include <QScrollBar>
 
@@ -64,6 +67,8 @@ MapView::MapView(QWidget *parent)
     // Adjustment for antialiasing is done by the items that need it
     setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing);
 
+    grabGesture(Qt::PinchGesture);
+
     connect(mZoomable, SIGNAL(scaleChanged(qreal)), SLOT(adjustScale(qreal)));
 }
 
@@ -131,6 +136,13 @@ bool MapView::event(QEvent *e)
             e->ignore();
             return false;
         }
+    } else if (e->type() == QEvent::Gesture) {
+        QGestureEvent *gestureEvent = static_cast<QGestureEvent *>(e);
+        if (QGesture *gesture = gestureEvent->gesture(Qt::PinchGesture)) {
+            QPinchGesture *pinch = static_cast<QPinchGesture *>(gesture);
+            if (pinch->changeFlags() & QPinchGesture::ScaleFactorChanged)
+                handlePinchGesture(pinch);
+        }
     }
 
     return QGraphicsView::event(e);
@@ -156,12 +168,7 @@ void MapView::wheelEvent(QWheelEvent *event)
 
         mZoomable->handleWheelDelta(event->delta());
 
-        // Place the last known mouse scene pos below the mouse again
-        QWidget *view = viewport();
-        QPointF viewCenterScenePos = mapToScene(view->rect().center());
-        QPointF mouseScenePos = mapToScene(view->mapFromGlobal(mLastMousePos));
-        QPointF diff = viewCenterScenePos - mouseScenePos;
-        centerOn(mLastMouseScenePos + diff);
+        adjustCenterFromMousePosition(mLastMousePos);
 
         // Restore the centering anchor
         setTransformationAnchor(QGraphicsView::AnchorViewCenter);
@@ -169,6 +176,10 @@ void MapView::wheelEvent(QWheelEvent *event)
     }
 
     QGraphicsView::wheelEvent(event);
+
+    // When scrolling the mouse does not move, but the view below it does.
+    // This affects the mouse scene position, which needs to be updated.
+    mLastMouseScenePos = mapToScene(viewport()->mapFromGlobal(mLastMousePos));
 }
 
 /**
@@ -217,3 +228,25 @@ void MapView::mouseMoveEvent(QMouseEvent *event)
     mLastMousePos = event->globalPos();
     mLastMouseScenePos = mapToScene(viewport()->mapFromGlobal(mLastMousePos));
 }
+
+void MapView::handlePinchGesture(QPinchGesture *pinch)
+{
+    setTransformationAnchor(QGraphicsView::NoAnchor);
+
+    mZoomable->handlePinchGesture(pinch);
+
+    QPoint centerPoint = pinch->hotSpot().toPoint();
+    adjustCenterFromMousePosition(centerPoint);
+
+    setTransformationAnchor(QGraphicsView::AnchorViewCenter);
+}
+
+void MapView::adjustCenterFromMousePosition(QPoint &mousePos)
+{
+    // Place the last known mouse scene pos below the mouse again
+    QWidget *view = viewport();
+    QPointF viewCenterScenePos = mapToScene(view->rect().center());
+    QPointF mouseScenePos = mapToScene(view->mapFromGlobal(mousePos));
+    QPointF diff = viewCenterScenePos - mouseScenePos;
+    centerOn(mLastMouseScenePos + diff);
+}
